package instance

import (
	"github.com/presslabs/controller-util/syncer"
	"k8s.io/apimachinery/pkg/labels"
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
)

type Instance interface {
	Meta
	GetComponents() map[int]Component
	GetOwner() interfaces.Object
	GetSettings() Settings
}

type Meta interface {
	meta.Meta
	//	GetName() string
	//	GetNamespace() string
	GetApplication() string
	GetInstance() string
	GetVersion() string
	GetComponent() string
	GetPartOf() string
	GetManagedBy() string
}

func InstanceLabels(m Meta) map[string]string {
	labels := labels.Set{
		"app.kubernetes.io/name":       m.GetApplication(),
		"app.kubernetes.io/instance":   m.GetInstance(),
		"app.kubernetes.io/version":    m.GetVersion(),
		"app.kubernetes.io/component":  m.GetComponent(),
		"app.kubernetes.io/part-of":    m.GetPartOf(),
		"app.kubernetes.io/managed-by": m.GetManagedBy(),
	}
	return labels
}

func NewSyncers(i Instance, r interfaces.Reconcile, owner interfaces.Object) (syncers []syncer.Interface) {
	components := i.GetComponents()

	//	labels := InstanceLabels(i)

	for _, c := range components {
		//	c.SetDefaults(i)

		mutators := c.GetObjects()

		for _, m := range mutators {

			//		m.SetLabels(labels)
			//		m.SetName(i.GetName())
			//		m.SetNamespace(i.GetNamespace())
			//	object.SetMeta(m, i)

			syncers = append(syncers, NewObjectSyncer(m, owner, r))
		}
	}

	return syncers
}
