package configmap_test

import (
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	. "k8s.libre.sh/objects/configmap"

	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"

	meta "k8s.libre.sh/meta"
	parameters "k8s.libre.sh/parameters"
)

var _ = Describe("ConfigMap", func() {
	var (
		//	cm         corev1.ConfigMap
		params     parameters.Parameters
		objectMeta *meta.ObjectMeta
		configMap  ConfigMap
	)

	BeforeEach(func() {
		//	cm := corev1.ConfigMap{}

		objectMeta = &meta.ObjectMeta{
			Name:      "test-cm",
			Namespace: "test-namespace",
		}

		params = parameters.Parameters{
			{
				Key:   "MY_PARAMETER",
				Value: "Parameter",
				Type:  "ConfigMap",
			},
		}

		configMap = ConfigMap{
			Meta:       objectMeta,
			Parameters: params,
		}

	})

	Describe("Mutate ConfigMap", func() {
		Context("With key value", func() {
			It("should generate env var", func() {

				expectedObj := &corev1.ConfigMap{
					ObjectMeta: metav1.ObjectMeta{
						Name:      "test-cm",
						Namespace: "test-namespace",
						//	Labels:
						//	Annotations:
					},
					Data: map[string]string{
						"MY_PARAMETER": "Parameter",
					},
				}

				cm := &corev1.ConfigMap{}
				err := configMap.Mutate(cm)

				//	res, err := paramEnvVar.GetEnvVar()
				Expect(err).NotTo(HaveOccurred())
				Expect(cm).To(Equal(expectedObj))
			})
		})
	})
})
