package service

import (
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	settings "k8s.libre.sh/parameters"

	corev1 "k8s.io/api/core/v1"
	"k8s.io/apimachinery/pkg/util/intstr"
)

// SecretRef represents a reference to a Secret
type SecretRef string

// Host represents a valid hostname
type Host string

// +kubebuilder:object:generate=true
type Service struct {
	*meta.ObjectMeta `json:"meta,omitempty"`
	*ServiceSpec     `json:",inline"`
}

// +kubebuilder:object:generate=true
type ServiceSpec struct {
	Ports []Port             `json:"ports,omitempty"`
	Type  corev1.ServiceType `json:"type,omitempty"`
}

type Port struct {
	Name     string          `json:"name,omitempty"`
	Port     int32           `json:"port,omitempty"`
	Protocol corev1.Protocol `json:"protocol,omitempty"`
	//	Ingress  Ingress         `json:"ingress,omitempty"`
}

// type Ingress struct {
//	Path string `json:"ingress,omitempty"`
//	Host Host   `json:"host,omitempty"`
//	TLS  bool   `json:"tls,omitempty"`
//}

func (o *Service) SetSettings(parameters settings.Parameters) {}

func (r *ServiceSpec) GetServiceType() corev1.ServiceType { return r.Type }

func (r *ServiceSpec) GetServicePorts() []corev1.ServicePort {

	ports := []corev1.ServicePort{}

	for _, p := range r.Ports {
		ports = append(ports, p.GetServicePort())
	}

	return ports
}

func (p *Port) GetServicePort() corev1.ServicePort {

	port := corev1.ServicePort{
		TargetPort: intstr.FromString(p.Name),
		Port:       p.Port,
		Name:       p.Name,
	}

	if len(p.Protocol) > 0 {
		port.Protocol = p.Protocol
	}

	return port
}

func (o *Service) Mutate(obj interfaces.Object) error {
	MutateService(o, obj.(*corev1.Service), o.ObjectMeta)
	meta.MutateMeta(o, obj)
	//	mutate.MutateMetaService(o, obj)
	return nil
}

func (o *Service) GetObject() interfaces.Object {
	return &corev1.Service{}
}
