/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package parameters

import (
	"reflect"

	"github.com/presslabs/controller-util/rand"
	corev1 "k8s.io/api/core/v1"
)

func (p *Parameters) GetData() map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		if param.Type == "Secret" || param.Type == "ConfigMap" {

			if len(param.Value) > 0 && len(param.Key) > 0 {
				data[param.Key] = param.Value

			}

			if len(param.Key) > 0 && len(param.Generate) > 0 {
				size := 12
				if param.Generate == GenerateRand24 {
					size = 24
				}
				random, err := rand.AlphaNumericString(size)
				if err != nil {
				}
				if len(data[param.Key]) == 0 {
					data[param.Key] = random
				}

			}
		}
	}
	return data
}

func (p *Parameters) GetEnv() []corev1.EnvVar {
	envVars := []corev1.EnvVar{}
	envVar := corev1.EnvVar{}
	var err error

	for _, param := range *p {
		envVar, err = param.GetEnvVar()

		if err != nil {
		}

		if len(envVar.Name) > 0 {
			envVars = append(envVars, envVar)
		}
	}

	return envVars

}

func (p *Parameters) GetEnvFrom() []corev1.EnvFromSource {
	envFroms := []corev1.EnvFromSource{}

	for _, param := range *p {
		if len(param.Source) > 0 && len(param.FromKey) == 0 && param.MountType == "envFrom" {

			envFrom, err := param.GetEnvFrom()

			if err == nil {
			}

			envFroms = append(envFroms, envFrom)

		}
	}

	envFroms = SliceUniqMap(envFroms)

	return envFroms
}

func SliceUniqMap(s []corev1.EnvFromSource) []corev1.EnvFromSource {
	var unique []corev1.EnvFromSource
	for _, v := range s {
		skip := false
		for _, u := range unique {
			if len(unique) == 0 {
				unique = append(unique, v)
			} else {
				if v.SecretRef != nil && u.SecretRef != nil && *v.SecretRef == *u.SecretRef {
					skip = true
					break
				}
				if v.ConfigMapRef != nil && u.ConfigMapRef != nil && *v.ConfigMapRef == *u.ConfigMapRef {
					skip = true
					break
				}
			}
		}
		if !skip {
			unique = append(unique, v)
		}
	}
	return unique
}

func Marshal(i interface{}) (Parameters, error) {
	parameters := Parameters{}

	t := reflect.TypeOf(i)

	ifv := reflect.ValueOf(i)

	for ii := 0; ii < t.NumField(); ii++ {
		f := t.Field(ii)

		fv := ifv.Field(ii)

		parameter, ok := fv.Interface().(Parameter)
		if !ok {
			//	return nil, ok
		}

		env, ok := f.Tag.Lookup("env")
		if !ok {
			//	return nil, ok
		}

		if len(parameter.Key) == 0 {
			parameter.Key = env
		}

		parameters = append(parameters, &parameter)
	}

	return parameters, nil
}
